





class Settings : 
    SavingPolicy  = dict(
        EmbeddedInstanceDesc = "Gameobjects" , 
        Atlas = 'Atlases' , 
        MaterialDesc  = 'Materials/{name}' , 
        CollectionDesc = 'Collections'
    )
    Templates = "https://github.com/MhadhbiXissam/defold-templates"
    defaultTemplate = "default"
